<?php

namespace Quivers\Integration\Model\Config\Source;

class ClaimingGroupConfig implements \Magento\Framework\Option\ArrayInterface
{
	protected $curlClient;
	protected $scopeConfig;
	protected $baseUrl;

	protected $baseTestUrl = "https://cloudhub.quiversdemo.com/";
	protected $baseProdUrl = "https://cloudhub-internal.quivers.com/";
	protected $claimingGroupApi = "/api/v1/private/ProductGroups/MyProductGroups?refId=";

    const XML_QUIVERS_API_KEY = 'quivers/parameters/quivers_api_key';
    const XML_QUIVERS_BUSINESS_REFID = 'quivers/parameters/merchant_ref_id';
    const XML_QUIVERS_SANDBOX = 'quivers/parameters/quivers_sanndbox';

	public function __construct(
		\Magento\Framework\HTTP\Client\Curl $curl,
        \Magento\Framework\App\Config\ScopeConfigInterface $scopeConfig
	) {
		$this->curlClient = $curl;
		$this->scopeConfig = $scopeConfig;
	}

	public function getClaimingGroup() {

        $storeScope = \Magento\Store\Model\ScopeInterface::SCOPE_STORE;
        $quiversAPIKey = $this->scopeConfig->getValue(self::XML_QUIVERS_API_KEY, $storeScope);
        $quiversSandbox = $this->scopeConfig->getValue(self::XML_QUIVERS_SANDBOX, $storeScope);
        $quiversBusinessRefId = $this->scopeConfig->getValue(self::XML_QUIVERS_BUSINESS_REFID, $storeScope);

		// get claimng group places by calling the claiming group place api
		if($quiversSandbox==1) $this->baseUrl = $this->baseTestUrl;
		else $this->baseUrl = $this->baseProdUrl;

		$profileUrl = $this->baseUrl.$this->claimingGroupApi.$quiversBusinessRefId;
        $this->curlClient->addHeader("Content-Type","application/json");
        $this->curlClient->addHeader("Authorization","apikey ".$quiversAPIKey);
        $this->curlClient->get($profileUrl);

		$response = json_decode($this->curlClient->getBody());
		$claimingGroups = [];
		if($this->curlClient->getStatus()==200 && is_array($response)) {
			if(count($response)>0) {
				foreach($response as $res) {
					if($res->Type=="Claiming") {
						array_push($claimingGroups, (object)['id' => $res->Id, 'name' => $res->Name ]);
					}
				}
			}
		}
		return $claimingGroups;
	}

	public function toOptionArray() {
		$allclaimingGroups = $this->getClaimingGroup();
		$options = [];
		if(count($allclaimingGroups)>0) {
			foreach($allclaimingGroups as $cGroup) {
				$options[] = ['value' => $cGroup->id, 'label' => __($cGroup->name)];
			}
		}
		return $options;
	}
}