define([
        'jquery',
        'Magento_Payment/js/view/payment/cc-form',
        'Magento_Checkout/js/model/totals',
        'PayPal'
    ],
    function ($, Component, totals, PayPal) {
        'use strict';
        var config=window.checkoutConfig.payment.quivers_integration;
        return Component.extend({
            defaults: {
                template: 'Quivers_Integration/payment/paypal-form',
                publicKey: config ? config.payPalClientId : '',
                sandbox: config ? config.payPalSandbox : true,
                title: config ? config.payPalTitle: 'PayPal'
            },

            initialize: function () {
                var self = this;
                this._super();
            },

            payPalClient: function() {
              if(this.sandbox == 1) {
                return {
                  sandbox: this.publicKey
                  }  
              }
              return {
                  production: this.publicKey,
                }
            },

            grandTotal: function() {
              // use base_grand_total because grand_total
              // doesn't have tax include
              return parseFloat(totals.totals().base_grand_total).toFixed(2);
            },

            currencyCode: function() {
              return totals.totals().quote_currency_code;
            },

            onPayPalButtonRender: function()
            {
              var total = this.grandTotal();
              var currency = this.currencyCode();
              var that = this;
              var payPalEnv = 'production';
              if(this.sandbox == 1) {
                payPalEnv = 'sandbox';
              }
              
              paypal.Button.render({
                env: payPalEnv,

                // PayPal Client IDs - replace with your own
                // Create a PayPal app: https://developer.paypal.com/developer/applications/create
                client: this.payPalClient(),

                style: {
                    label: 'pay',
                    size:  'medium', // small | medium | large | responsive
                    shape: 'rect',   // pill | rect
                    color: 'blue',   // gold | blue | silver | black
                    tagline: false
                },

                // payment() is called when the button is clicked
                payment: function(data, actions) {
                  // Make a call to the REST api to create the payment
                    return actions.payment.create({
                        payment: {
                          intent: 'authorize',
                            transactions: [
                                {
                                    amount: { 
                                      total: that.grandTotal(),
                                      currency: that.currencyCode()
                                    }
                                }
                            ]
                        }
                    });
                },

                // onAuthorize() is called when the buyer approves the payment
                onAuthorize: function(data, actions) {
                    return that.handlePaymentData(data, actions)
                }
              }, '#paypal_button');

            },

            handlePaymentData: function(data) {
              this.setPaymentId(data.paymentID);
              this.setPayerId(data.payerID);
              this.placeOrder();
            },

            context: function() {
                return this;
            },

            isPlaceOrderActionAllowed: function() {
                return true;
            },

            getPaymentId: function(value) {
                var payment_method_code = this.getCode();
                var element = $('#' + payment_method_code + '-payment-id');
                return element.val();
            },

            setPaymentId: function(value) {
                var payment_method_code = this.getCode();
                var element = $('#' + payment_method_code + '-payment-id');
                element.val(value);
            },

            getPayerId: function(value) {
                var payment_method_code = this.getCode();
                var element = $('#' + payment_method_code + '-payer-id');
                return element.val();
            },

            setPayerId: function(value) {
                var payment_method_code = this.getCode();
                var element = $('#' + payment_method_code + '-payer-id');
                element.val(value);
            },

            getData: function()
            {
                var data = {
                    'method': this.getCode(),
                };
                data.additional_data = {
                        'cc_paypal_payment_id': this.getPaymentId(),
                        'cc_paypal_payer_id': this.getPayerId(),
                    };

                return data;
            },

            getTitle: function() {
                return this.title;
            },

            getCode: function() {
                return 'quivers_integration_paypal';
            },

        });
    }
);
