define([
        'jquery',
        'Magento_Payment/js/view/payment/cc-form',
        'Stripe',
        'Magento_Checkout/js/model/quote',
        'Magento_Checkout/js/checkout-data'
    ],
    function ($, Component, stripe, quote, checkoutdata) {
        'use strict';
        var config=window.checkoutConfig.payment.quivers_integration;

        return Component.extend({
            defaults: {
                template: 'Quivers_Integration/payment/stripe-form',
                publicKey: config ? config.publicKey : '',
                siteKey: config ? config.siteKey : '',
                title: config ? config.title: 'Pay By Card',
                storedCards: config ? config.stripeCards: [],
                selectedCard: config ? config.selectedCard : [],
                customerId: config ? config.customerId : [] ,
                baseUrl: config ? config.baseUrl: '',
                paymentmethods: config ? config.paymentmethods: [],
                path: config ? config.path: ''
            },

            initialize: function () {
                var self = this;

                this._super();

                // Initialize Stripe object thinger
                this.stripe = Stripe(this.publicKey);
                this.elements = this.stripe.elements();
            },

            onCardElementRender: function()
            {   
                var me =this;
                var sitekey = this.siteKey?this.siteKey:'SITE_KEY_NOT_THERE';
                var style = {
                base: {
                    // Add your base input styles here. For example:
                    fontSize: '16px',
                    color: "#32325d",
                  }
                };

                // Create an instance of the card Element.
                this.card = this.elements.create('card', {style: style, hidePostalCode: true});
                // Add an instance of the card Element into the `card-element` <div>.
                this.card.mount('#card-element');
                var button = document.getElementById("action primary checkout button");
                button.disabled = false;
                this.card.addEventListener('change', function(event) {
                    var displayError = document.getElementById('card-errors');
                    if (event.error) {
                        displayError.textContent = event.error.message;
                        button.disabled = true;
                    } else {
                        button.disabled = false;
                        displayError.textContent = '';
                    }
                });
                grecaptcha.ready(function() {
                        grecaptcha.render("captcha_box", {
                            "sitekey": sitekey,
                            "callback" :function() {
                                var response = grecaptcha.getResponse();
                                if(response.length != 0){
                                    jQuery('#form-submit').trigger('click');
                                }else{
                                    grecaptcha.execute();
                                }
                            },
                            "size" : 'invisible'
                        });
                    });
            },
            context: function() {
                return this;
            },

            getCode: function() {
                return 'quivers_integration';
            },

            isActive: function() {
                return true;
            },
            
            isPlaceOrderActionAllowed: function() {
                return true;
            },

            handleStripeToken: function (result) {
                if (result.error) {
                    this.setToken(null);
                    var errorElement = document.getElementById('card-errors');
                    errorElement.textContent = result.error.message;
                    setTimeout(function(){window.location.reload(true);}, 5000);
                } else {
                    this.setToken(result.source.id);
                    this.placeOrder()
                }
            },

            handle3dStripe: function (result) {
                if (result.error) 
                {   jQuery('body').loader('destroy');
                    this.setToken(null);
                    var errorElement = document.getElementById('card-errors');
                    var check = result.error.message.search("authenticate");
                    if(check==-1)
                    {
                        var errorMessage=result.error.message;
                    }
                    else
                    {   
                        var error = "If the issue persists please contact Quivers at 805-250-4555.";
                        var errorMessage=result.error.message+error;
                    }
                    errorElement.textContent = errorMessage;
                    setTimeout(function(){window.location.reload(true);}, 5000);
                } 
                else 
                {   var paymentSourceId = jQuery('#'+this.getCode()+'-card-id').children(':selected').attr('value');
                    if(paymentSourceId)
                    {
                        this.placeOrder();
                    }
                    else
                    {
                        
                        var address = checkoutdata.getNewCustomerBillingAddress();
                        if (address == null) {
                            address = quote.shippingAddress();
                        }
                        var postcode = (address.postcode);
                        var payment_method_code = this.getCode();
                        this.stripe.createSource(this.card, {owner: {address: { postal_code: postcode }}}).then(
                            this.handleStripeToken.bind(this));
                    }
                }

            },

            getToken: function(value) {
                var payment_method_code = this.getCode();
                var element = $('#' + payment_method_code + '-token');
                return element.val();
            },

            setToken: function(value) {
                var payment_method_code = this.getCode();
                var element = $('#' + payment_method_code + '-token');
                element.val(value);
            },

            placeOrder: function() {
                var paymentSourceId = jQuery('#'+this.getCode()+'-card-id').children(':selected').attr('value');
                
                if(this.getToken()) {
                    this._super();
                }else if(paymentSourceId){
                    this.setToken(paymentSourceId);
                    this.placeOrder();
                }else {
                    var address = checkoutdata.getNewCustomerBillingAddress();
                    if (address == null) {
                        address = quote.shippingAddress();
                    }
                    var postcode = (address.postcode);
                    var payment_method_code = this.getCode();
                    this.stripe.createSource(this.card, {owner: {address: { postal_code: postcode }}}).then(
                        this.handleStripeToken.bind(this)
                    );
                }
                return false;
            },

            onPlaceOrderClick: function() {
                jQuery('body').loader('show');
                var me =this;
                var paymentSourceId = jQuery('#'+this.getCode()+'-card-id').children(':selected').attr('value');
                var address = checkoutdata.getNewCustomerBillingAddress();
                if (address == null) {
                        address = quote.shippingAddress();
                }
                if(paymentSourceId){
                    this.setToken(paymentSourceId);
                    var data=this.getSelectedCardFromList()[0].childNodes[0].data.split(" ");
                    var last4=data[0].split("-")[3];
                    var exp_month=data[1].split("/")[0];
                    var exp_year=data[1].split("/")[1];
                    for(var i = 0 ; i < this.paymentmethods.length; i++) {
                        if(this.paymentmethods[i]["exp_month"]==exp_month && this.paymentmethods[i]["exp_year"]==exp_year && 
                            this.paymentmethods[i]["last4"]==last4){

                            var payment_method = this.paymentmethods[i]["payment_method_id"];
                            break;
                        }
                    }
                    $.get(this.baseUrl+this.path, {},function(response){
                    if (!response || !response.client_secret) {
                        jQuery('body').loader('destroy');
                        me.setToken(null);
                        var errorElement = document.getElementById('card-errors');
                        var error = "Error processing Payments by Card.\
                        Please choose a different payment method and try again.\
                        If the issue persists please contact Quivers at 805-250-4555."
                        errorElement.textContent = error;
                        setTimeout(function(){window.location.reload(true);}, 5000);
                        return;
                    }
                    var data = {
                    payment_method: payment_method
                    };
                    me.stripe.confirmCardPayment(response.client_secret,data).then(
                    me.handle3dStripe.bind(me));
                    });
                }
                else{
                var address = checkoutdata.getNewCustomerBillingAddress();
                if (address == null) {
                    address = quote.shippingAddress();
                }
                var postcode = (address.postcode);
                var payment_method_code = this.getCode();
                var data = {
                    payment_method: {
                    card: this.card
                   }
                };
                $.get(this.baseUrl+this.path, {},function(response){
                if (!response || !response.client_secret) {
                    jQuery('body').loader('destroy');
                    me.setToken(null);
                    var errorElement = document.getElementById('card-errors');
                    var error = "Error processing Payments by Card.\
                    Please choose a different payment method and try again.\
                    If the issue persists please contact Quivers at 805-250-4555."
                    errorElement.textContent = error;
                    setTimeout(function(){window.location.reload(true);}, 5000);
                    return;
                }
                me.stripe.confirmCardPayment(response.client_secret,data).then(
                me.handle3dStripe.bind(me));
                });
            }},
            getcaptcha: function () {
                    grecaptcha.execute().then(response => {
                        }).catch(e => {
                            jQuery('#form-submit').trigger('click');
                        });
            },
            getTitle: function() {
                return this.title;
            },

            getData: function()
            {
                var data = {
                    'method': this.getCode(),
                };
                data.additional_data = {
                        'cc_stripejs_token': this.getToken(),
                        'captcha_token': grecaptcha.getResponse()?  grecaptcha.getResponse():null,
                        'cc_save_card': jQuery('#save_my_card_checkbox input').attr('checked'),
                    };

                return data;
            },

            getStoredCards: function() {
                return this.storedCards;
            },

            getSelectedCard: function() {
                return this.selectedCard.label;
            },

            getCardSelectVisibility: function() {
                return this.storedCards.length
            },

            getSelectedCardFromList: function() {
                var selected = jQuery('#'+this.getCode()+'-card-id').find(":selected");
                return selected;
            },

            getFormElementVisibility: function() {
                var selected = this.getSelectedCardFromList();
                if(!this.getCardSelectVisibility() || !selected.attr('value')) return 1;
                else return 0;
            },

            getSaveCardElementVisibility: function() {
                if(!this.customerId) return 0;
                return this.getFormElementVisibility();
            },

            selectCardAfterRender: function() {
                //handle dropdown changes in card element
                var self = this;
                var cardElement = jQuery('#'+this.getCode()+'-card-id');
                cardElement.on('change', function() {
                    if(cardElement.children(':selected').attr('value')) {
                        jQuery('#'+self.getCode()+'-form').css('display', 'none');
                        jQuery('#save_my_card_checkbox').css('display', 'none');
                    }
                    else {
                        jQuery('#'+self.getCode()+'-form').css('display', 'block');
                        jQuery('#save_my_card_checkbox').css('display', 'block');                        
                    }
                });
            },
        });
    }
);
